library(ape)
library(phangorn)

lower <- c(cal1 = 0.258, cal2 = 0.691, cal3 = 1.81, cal4 = 0.93, cal5 = 0.392, cal6 = 3.444,
           cal7 = 1.26, cal8 = 0.61, cal9 = 2.00, cal10 = 3.05, cal11 = 2.447, cal12 = 2.00,
           cal13 = 2.476)
           # , cal16 = 5.588)

upper <- c(cal1 = 2.3573, cal2 = 3.3676, cal3 = 5.7540, cal4 = 4.4800, cal5 = 2.2547, cal6 = 6.9883,
           cal7 = 3.7798, cal8 = 2.8369, cal9 = 4.8308, cal10 = 5.9076, cal11 = 4.8950, cal12 = 6.8426,
           cal13 = 6.8832)
           # , cal16 = 8.2063)

cvalues <- ((upper/lower) - 1)/12.706204736175

refTree <- read.tree("/Users/David/Grive/Slater_Lab/Charadriiformes/Dating/baseml.tre")

clades <- list()
clades[[1]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Alcidae_Uria_aalge", "Alcidae_Uria_lomvia")), type = "tips"))]
clades[[2]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Alcidae_Alca_torda", "Alcidae_Pinguinus_impennis")), type = "tips"))]
clades[[3]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Alcidae_Alca_torda", "Alcidae_Fratercula_arctica")), type = "tips"))]
clades[[4]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Alcidae_Fratercula_arctica", "Alcidae_Aethia_psittacula")), type = "tips"))]
clades[[5]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Alcidae_Fratercula_arctica", "Alcidae_Fratercula_corniculata")), type = "tips"))]
clades[[6]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Alcidae_Alca_torda", "Stercorariidae_Stercorarius_chilensis")), type = "tips"))]
clades[[7]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Scolopacidae_Arenaria_interpres", "Scolopacidae_Calidris_canutus")), type = "tips"))]
clades[[8]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Scolopacidae_Gallinago_gallinago", "Scolopacidae_Coenocorypha_pusilla")), type = "tips"))]
clades[[9]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Scolopacidae_Gallinago_gallinago", "Scolopacidae_Numenius_minutus")), type = "tips"))]
clades[[10]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Jacanidae_Jacana_jacana", "Rostratulidae_Rostratula_benghalensis")), type = "tips"))]
clades[[11]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Pedionomidae_Pedionomus_torquatus", "Thinocoridae_Thinocorus_orbignyianus")), type = "tips"))]
clades[[12]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Haematopodidae_Haematopus_ater", "Recurvirostridae_Recurvirostra_avosetta")), type = "tips"))]
clades[[13]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Chionidae_Chionis_minor", "Burhinidae_Burhinus_capensis")), type = "tips"))]
# clades[[14]] <- refTree$tip.label[unlist(Descendants(refTree, getMRCA(refTree, c("Chionidae_Chionis_minor", "Laridae_Larus_canus")), type = "tips"))]

title <- c("italic('Uria aalge') + italic('U. lomvia')",
           "italic('Alca') + italic('Pinguinus')",
           "Alcidae",
           "Fraterculini + Aethiini",
           "italic('Fratercula arctica') + italic('F. corniculata')",
           "Alcoidea",
           "Arenariinae",
           "italic('Gallinago') + italic('Coenocorypha')",
           "Scolopacidae",
           "Jacanoidea",
           "Thinocoroidea",
           "Haematopodoidea",
           "Chionida")
           # "Charadriiformes ~ (root)")

xlimmin <- c(rep(0, 2), 1, rep(0, 2), 3, 1, 0, 1, rep(2, 2), 1, 2)
             # , 5)
xlimmax <- c(3, 4, 7, 6, 3, 8, 5, 4, 6, 7, 6, rep(8, 2))
             #, 9)

# A function that is solely copied over to not have to load MCMCTreeR and deal with its dependencies. Note that the 'commonAnc' function on line 65 was replaced here by getMRCA()
# from ape, which does exactly the same thing.
nodeToPhy <- function (phy, monoGroups, nodeCon, returnPhy = T)
{
    if (is(monoGroups)[1] != "list")
        monoGroups <- list(monoGroups)
    for (x in 1:length(nodeCon)) {
        nodeLocation <- which((Ntip(phy) + 1):(Ntip(phy) + Nnode(phy)) == getMRCA(phy, monoGroups[[x]]))
        phy$node.labels[nodeLocation] <- nodeCon[x]
    }
    if (returnPhy == T)
        return(phy)
    if (returnPhy == F) {
        s <- write.tree(phy)
        s <- gsub(")NA", ")", s)
        s <- gsub("\\[", "(", s)
        s <- gsub("\\]", ")", s)
        s <- gsub("~", ",", s)
        names(s) <- NULL
        treeFile <- data.frame(cbind(paste(Ntip(phy), "1"), s, "//end of file"))
        colnames(treeFile) <- rep("", 3)
        return(treeFile)
    }
}

# This function from MCMCTreeR has only been modified to fix the issue reported at http://github.com/PuttickMacroevolution/MCMCtreeR/issues/4 (in effect when estimateScale == T).
# However, we mostly put it here just so that we do not have to load the MCMCTreeR package, which has a dependency ('sn') that is failing to install.
estimateCauchy <- function (minAge, maxAge, phy, monoGroups, scale = 0.2, offset = 0.1, estimateScale = T, minProb = 0, maxProb = 0.975, plot = FALSE, pdfOutput = "cauchyPlot.pdf",
                            writeMCMCTree = FALSE, mcmcTreeName = "cauchyInput.tre")
{
    lMin <- length(minAge)
    lMax <- length(maxAge)
    if (lMin != lMax)
        stop("length of ages do not match")
    if (length(minProb) < lMin) {
        minProb <- rep_len(minProb, lMin)
        print("warning - minProb parameter value recycled")
    }
    if (length(maxProb) < lMin) {
        maxProb <- rep_len(maxProb, lMin)
        print("warning - maxProb parameter value recycled")
    }
    if (length(offset) < lMin) {
        offset <- rep_len(offset, lMin)
        print("warning - offset parameter value recycled")
    }
    if (length(scale) < lMin) {
        scale <- rep_len(scale, lMin)
        print("warning - scale parameter value recycled")
    }
    if (length(estimateScale) < lMin) {
        estimateScale <- rep_len(estimateScale, lMin)
        print("warning - estimateScale argument recycled")
    }
    nodeFun <- function(x) {
        scaleInt <- scale[x]
        locationInt <- minAge[x]
        offsetInt <- offset[x]
        estimateScaleInt <- estimateScale[x]
        maxProbInt <- maxProb[x]
        minProbInt <- minProb[x]
        if (estimateScaleInt == F) {
            scaleInt <- scaleInt
            offsetInt <- offsetInt
        }
        else {
            p <- offsetInt
            cEsts <- c()
            cTest <- seq(0.001, 10, by = 0.001)
            # Modified; see the GitHub issue link above
            for (u in 1:length(cTest)) cEsts[u] = minAge[x] * (1 + p + cTest[u] * 1/tan((pi * (0.5 + 1/pi * atan(p/cTest[u])) * (1 - maxProbInt)/(1 - minProbInt))))
            closest <- which(abs(cEsts - maxAge[x]) == min(abs(cEsts - maxAge[x])))
            upperEst <- cEsts[closest]
            scaleInt <- cTest[closest]
        }
        if (minProbInt < 1e-07)
            minProbInt <- 1e-300
        nodeCon <- paste0("'L[", locationInt, "~", offsetInt, "~", scaleInt, "~", minProbInt, "]'")
        parameters <- c(locationInt, offsetInt, scaleInt, minProbInt)
        names(parameters) <- c("tL", "p", "c", "pL")
        return(list(nodeCon, parameters))
    }
    out <- sapply(1:lMin, nodeFun)
    output <- c()
    prm <- matrix(unlist(out[2, ]), ncol = 4, byrow = T)
    rownames(prm) <- paste0("node_", 1:lMin)
    colnames(prm) <- c("tL", "p", "c", "pL")
    output$parameters <- prm
    output$apePhy <- nodeToPhy(phy, monoGroups, nodeCon = unlist(out[1, ]), T)
    output$mcmctree <- nodeToPhy(phy, monoGroups, nodeCon = unlist(out[1, ]), F)
    if (writeMCMCTree == T) {
        outputTree <- nodeToPhy(phy, monoGroups, nodeCon = unlist(out[1, ]), returnPhy = F)
        write.table(outputTree, paste0(mcmcTreeName), quote = F, row.names = F, col.names = F)
    }
    if (plot == T) {
        cat("warning - cauchy plots will be approximations!")
        if (length(list.files(pattern = paste0(pdfOutput))) != 0) {
            cat(paste0("warning - deleting and over-writing file ", pdfOutput))
            file.remove(paste0(pdfOutput))
        }
        pdf(paste0(pdfOutput), family = "Times")
        for (k in 1:dim(prm)[1]) {
            plotMCMCTree(prm[k, ], method = "cauchy", paste0(rownames(prm)[k], " cauchy"), upperTime = max(maxAge) + 1)
        }
        dev.off()
    }
    output$nodeLabels <- unlist(out[1, ])
    return(output)
}

# Copied over without change from MCMCTreeR
cauchyMCMCTree <- function (xRange = c(0, 2), tL = 1, p = 0.5, c = 0.2, minProb = 0.025, maxProb = 0.975)
{
    A = 0.5 + (1/pi) * atan(p/c)
    omega <- (maxProb/minProb) * (1/(pi * A * c * (1 + (p/c)^2)))
    xx <- seq(xRange[1], xRange[2], by = 0.001)
    rc <- 1
    numero <- c()
    for (x in xx) {
        t <- x
        if (t > tL)
            numero[rc] <- maxProb * (1/(A * pi * c * tL * (1 + ((t - tL * (1 + p))/(c * tL))^2)))
        if (t <= tL)
            numero[rc] <- minProb * (omega/tL) * (t/tL)^(omega - 1)
        rc <- rc + 1
    }
    time <- xx
    density <- numero
    return(cbind(time, density))
}

# To make the plot titles work, we need to slightly modify the `plotMCMCTree` function, so that main = title rather than main = paste(title).
# While we are at it, we also improve the graphics a bit, and simplify the function to focus solely on Cauchy densities.
plotCauchy <- function (parameters, title, upperTime, lowerTime = 0, plotMCMCTreeData = TRUE)
{
    timeScale <- c(lowerTime, upperTime)
    if (plotMCMCTreeData == TRUE) {
        plot(cauchyMCMCTree(xRange = timeScale, tL = parameters[1], p = parameters[2], c = parameters[3], minProb = parameters[4]), type = "l", lty = 1, col = "grey20",
             xlim = timeScale, lwd = 1, xpd = T, xaxs = "i", yaxs = "i", bty = "l", las = 1, ylab = "density", xlab = "time units (10 Ma)", main = title, cex = 1)
        polygon(cauchyMCMCTree(xRange = timeScale, tL = parameters[1], p = parameters[2], c = parameters[3], minProb = parameters[4]),
                col = adjustcolor("lightblue", alpha.f = 0.3), border = "blue")
    }
    else {
        out <- cauchyMCMCTree(xRange = timeScale, tL = parameters[1], p = parameters[2], c = parameters[3], minProb = parameters[4])
        return(out)
    }
}

cauchy_results <- estimateCauchy(minAge = lower, maxAge = upper, phy = refTree, monoGroups = clades, scale = cvalues,
                                 offset = rep(0, length(lower)), estimateScale = rep(F, length(lower)), minProb = rep(0, length(lower)),
                                 maxProb = rep(0.95, length(lower)), plot = F)

png(filename = "/Users/David/Grive/Slater_Lab/Charadriiformes/Calibrations/cauchy_cals.png", width = 1800, height = 2400, pointsize = 24)
layout(matrix(1:15, nrow = 5, ncol = 3, byrow = T))
for(i in 1:length(lower)) {
    plotCauchy(cauchy_results$parameters[i,], title = parse(text = title[i]), upperTime = xlimmax[i], lowerTime = xlimmin[i])
}
dev.off()
